@@ -21,12 +21,13 @@ Scan the resource providers, add some metadata and store it in the database
 """
 
 from elisa.core import common
+from elisa.core.utils.locale_helper import system_encoding
 from elisa.core.media_uri import MediaUri
 
-from twisted.internet import task, defer, reactor
+from twisted.internet import task, defer, reactor, threads
 from twisted.python.failure import Failure
 
-import time
+import time, os
 
 from elisa.core.components.resource_provider import ResourceProvider
 
@@ -140,8 +141,9 @@ class MediaScanner(ResourceProvider):
         self.store = store
 
         dfr.addCallback(lambda x: self.create_schema())
-        # FIXME: add configuration system
-        dfr.addCallback(lambda x: DatabaseParser.create({}))
+        # FIXME: add configuration system. Passing through my configuration is
+        # not exactly nice.
+        dfr.addCallback(lambda x: DatabaseParser.create(self.config))
         dfr.addCallback(set_parser)
         dfr.addCallback(lambda x: LocalResource.create({}))
         dfr.addCallback(set_local_resource)
@@ -267,7 +269,9 @@ class MediaScanner(ResourceProvider):
             source_uri = source_uri.join('')
 
         def start_scanning(result, source_uri):
-            s = ScanResource(source_uri)
+            name = os.path.basename(os.path.dirname(source_uri.path))
+            s = ScanResource(source_uri, name)
+            s.name
             self.scan_stat.queue.put(s)
 
             if not self.running:
@@ -282,25 +286,27 @@ class MediaScanner(ResourceProvider):
     def _update_stat(self, result, model, stat):
         stat.files_scanned += 1
 
-        if isinstance(result, Failure):
-            stat.files_failed.append( (model.uri, result) )
+        # deactivated as we don't use it atm and it leaks memory
+        #if isinstance(result, Failure):
+        #    stat.files_failed.append( (model.uri, result) )
+
+    def _count_files(self, path):
+        path = os.path.abspath(path)
+
+        def iterate(path):
+            counter = 0
+            for root, dirs, files in os.walk(path.encode(system_encoding())):
+                # count only the files
+                counter += len(files)
+            return counter
+
+        dfr = threads.deferToThread(iterate, path)
+        return dfr
 
     def _file_found(self, model, stat):
-        stat.files_total += 1
-
-        # make a pause to take a breath
-        delay_dfr = defer.Deferred()
-
-        def delay_callback(result, delay_dfr):
-            self._current_delay = reactor.callLater(
-                float(self.config.get('delay')), delay_dfr.callback, result)
-            return result
-
         dfr = self.parser.query_model(model, stat)
         dfr.addBoth(self._update_stat, model, stat)
-        dfr.addBoth(delay_callback, delay_dfr)
-
-        return delay_dfr
+        return dfr
 
     def _scan(self):
 
@@ -329,12 +335,19 @@ class MediaScanner(ResourceProvider):
 
         # clear stat
         scan_resource.last_scan = time.time()
-        scan_resource.files = 0
-        scan_resource.dirs = 0
+        scan_resource.files_scanned = 0
+        scan_resource.files_total = 0
         scan_resource.state = ScanResource.SCANNING_FS
 
-        return self._scan_recursive(scan_resource).addCallback(run_next, scan_resource)
+        def set_total_files(result, scan_resource):
+            scan_resource.files_total = result
+            return self._scan_recursive(scan_resource)
 
+        dfr = self._count_files(scan_resource.root_uri.path)
+        dfr.addCallback(set_total_files, scan_resource)
+        dfr.addCallback(run_next, scan_resource)
+
+        return dfr
 
     def _scan_recursive(self, scan_resource):
         #resources = common.application.resource_manager
