@@ -41,9 +41,16 @@ from elisa.plugins.poblesec.base.coverflow import \
 from elisa.plugins.poblesec.base.grid import GridItemGridController
 
 from elisa.plugins.poblesec.actions import Action
-from elisa.plugins.poblesec.actions import PlayAllAction, ShuffleAction
-from elisa.plugins.poblesec.actions import AddToFavoritesAction, AddToPlaylistAction
-from elisa.plugins.poblesec.actions import LinkAction
+from elisa.plugins.database.actions import ArtistPlayAllAction, \
+                                           AlbumPlayAllAction, \
+                                           TrackPlayAllAction
+from elisa.plugins.database.actions import ArtistShuffleAction, \
+                                           AlbumShuffleAction, \
+                                           TrackShuffleAction
+from elisa.plugins.database.actions import AlbumAddToFavoritesAction, \
+                                           TrackAddToFavoritesAction, \
+                                           PhotoAddToFavoritesAction
+from elisa.plugins.database.actions import ViewPhotoAlbumSlideshowAction
 
 from elisa.plugins.favorites.models import FavoritesItem
 
@@ -111,135 +118,6 @@ def music_lib_decades_decorator(controller):
     return defer.succeed(None)
 
 
-class ArtistPlayAllAction(PlayAllAction):
-
-    def get_items(self, item):
-        dfr = item.tracks.find()
-        dfr.addCallback(lambda rs: rs.all())
-        dfr.addCallback(lambda items: sorted(items, key=lambda a: a.file_path.lower()))
-        return dfr
-
-class AlbumPlayAllAction(PlayAllAction):
-
-    def get_items(self, item):
-        # Hack to skip the dummy "All Tracks" album
-        if hasattr(item, 'artist'):
-            return defer.succeed([])
-
-        dfr = item.tracks.find()
-        dfr.addCallback(lambda rs: rs.all())
-        dfr.addCallback(lambda items: sorted(items, key=lambda a: a.file_path.lower()))
-        return dfr
-
-
-class TrackPlayAllAction(PlayAllAction):
-
-    def get_items(self, item):
-        return defer.succeed([item])
-
-    def run(self):
-        track = self.controller.track
-        if track is not None:
-            playable_model = PlayableModel()
-            playable_model.uri = MediaUri('file://' + track.file_path)
-            playable_model.title = track.title
-            # Enqueue the following tracks
-            tracks = self.controller.tracks
-            following_tracks = [playable_model]
-            index = tracks.index(track)
-            for ftrack in tracks[index+1:]:
-                playable_model = PlayableModel()
-                playable_model.uri = MediaUri('file://' + ftrack.file_path)
-                playable_model.title = ftrack.title
-                following_tracks.append(playable_model)
-            self.play(following_tracks)
-            dfr = defer.succeed(True)
-        else:
-            dfr = super(TrackPlayAllAction, self).run()
-
-        return dfr
-
-
-class ArtistShuffleAction(ArtistPlayAllAction, ShuffleAction):
-    pass
-
-class AlbumShuffleAction(AlbumPlayAllAction, ShuffleAction):
-    pass
-
-class TrackShuffleAction(TrackPlayAllAction, ShuffleAction):
-    pass
-
-
-class AlbumAddToFavoritesAction(AddToFavoritesAction):
-
-    def run(self):
-        if self.toggled:
-            if self.controller.artist:
-                # FIXME: add AND condition on foreign_class
-                dfr = common.application.store.find(FavoritesItem,
-                                                    FavoritesItem.foreign_id == self.controller.artist.name)
-                dfr.addCallback(lambda rs: rs.one())
-                dfr.addCallback(self.remove)
-        else:
-            favorite = FavoritesItem()
-            favorite.added_time = datetime.datetime.now()
-
-            # expecting an AlbumsController
-            if self.controller.artist:
-                favorite.foreign_id = self.controller.artist.name
-                favorite.foreign_class = u'Artist'
-                favorite.title = self.controller.artist.name
-                self.store(favorite)
-
-class TrackAddToFavoritesAction(AddToFavoritesAction):
-
-    def run(self):
-        if self.toggled:
-            if self.controller.album:
-                # FIXME: add AND condition on foreign_class
-                dfr = common.application.store.find(FavoritesItem,
-                                                    FavoritesItem.foreign_id == self.controller.album.name)
-                dfr.addCallback(lambda rs: rs.one())
-                dfr.addCallback(self.remove)
-            elif self.controller.track:
-                # FIXME: add AND condition on foreign_class
-                dfr = common.application.store.find(FavoritesItem,
-                                                    FavoritesItem.foreign_id == self.controller.track.file_path)
-                dfr.addCallback(lambda rs: rs.one())
-                dfr.addCallback(self.remove)
-        else:
-            favorite = FavoritesItem()
-            favorite.added_time = datetime.datetime.now()
-
-            # expecting an TracksController
-            if self.controller.album:
-                favorite.foreign_id = self.controller.album.name
-                favorite.foreign_class = u'MusicAlbum'
-                favorite.title = self.controller.album.name
-                self.store(favorite)
-            elif self.controller.track:
-                favorite.foreign_id = self.controller.track.file_path
-                favorite.foreign_class = u'MusicTrack'
-                favorite.title = self.controller.track.title
-                self.store(favorite)
-
-
-class ArtistAddToPlaylistAction(AddToPlaylistAction):
-
-    def run(self):
-        raise NotImplementedError()
-
-class AlbumAddToPlaylistAction(AddToPlaylistAction):
-
-    def run(self):
-        raise NotImplementedError()
-
-class TrackAddToPlaylistAction(AddToPlaylistAction):
-
-    def run(self):
-        raise NotImplementedError()
-
-
 class GenericArtistsController(HierarchyController):
 
     def initialize(self):
@@ -336,7 +214,7 @@ class ArtistsDbPreviewListController(ArtistsDbController, MenuItemPreviewListCon
 
     def item_to_label(self, item):
         if isinstance(item, Action):
-            return item.title or '#'
+            return '#'
         return item.name
 
 class ArtistsDbCoverflowController(ArtistsDbController, ImageWithReflectionCoverflowController):
@@ -366,26 +244,26 @@ class GenericAlbumsDbController(HierarchyController):
             action = AlbumPlayAllAction(self)
             action.title = _('Play All')
             # FIXME: there is no two lines support in AlbumController
-            action.subtitle = "%s '%s'" % (_('Play All Tracks By'), self.artist.name)
+            action.subtitle = _("Play All Tracks By '%s'") % self.artist.name
             actions.append(action)
 
             action = AlbumShuffleAction(self)
             action.title = _('Shuffle & Play')
             # FIXME: there is no two lines support in AlbumController
-            action.subtitle = "%s '%s'" % (_('Shuffle & Play All Tracks By'), self.artist.name)
+            action.subtitle = _("Shuffle & Play All Tracks By '%s'") % self.artist.name
             actions.append(action)
 
             action = AlbumAddToFavoritesAction(self)
             # FIXME: there is no two lines support in AlbumController
             action.untoggled_title = _('Add To Favorites')
-            action.untoggled_subtitle = "%s '%s' %s" % (_('Add The Artist'), self.artist.name, _('To Your Favorites'))
+            action.untoggled_subtitle = _("Add The Artist '%s' To Your Favorites") % self.artist.name
             action.toggled_title = _('Remove From Favorites')
-            action.toggled_subtitle = "%s '%s' %s" % (_('Remove The Artist'), self.artist.name, _('From Your Favorites'))
+            action.toggled_subtitle = _("Remove The Artist '%s' From Your Favorites") % self.artist.name
             actions.append(action)
 
-            # FIXME: add AND condition on foreign_class
             dfr = common.application.store.find(FavoritesItem,
-                                                FavoritesItem.foreign_id == self.artist.name)
+                                                FavoritesItem.foreign_id == self.artist.name,
+                                                FavoritesItem.foreign_class == u'Artist')
             dfr.addCallback(lambda rs: rs.all())
             dfr.addCallback(action.setup)
         else:
@@ -414,7 +292,6 @@ class GenericAlbumsDbController(HierarchyController):
 
             if isinstance(item, Action):
                 dfr = item.run()
-
             elif hasattr(item, 'artist'):
                 # Dummy "All Tracks" album
                 browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
@@ -550,6 +427,11 @@ class GenericTracksDbController(HierarchyController):
         self.tracks = tracks
         self.artist = artist
 
+        # we don't need the fastscroller, because we're going to order tracks
+        # by track number
+        if album:
+            self.fastscroller = False
+
         dfr= super(GenericTracksDbController, self).initialize()
 
         return dfr
@@ -565,42 +447,42 @@ class GenericTracksDbController(HierarchyController):
         if self.album:
             action = TrackPlayAllAction(self)
             action.title = _('Play All')
-            action.subtitle = "%s '%s'" % (_('Play All Tracks From'), self.album.name)
+            action.subtitle = _("Play All Tracks From '%s'") % self.album.name
             actions.append(action)
 
             action = TrackShuffleAction(self)
             action.title = _('Shuffle & Play')
-            action.subtitle = "%s '%s'" % (_('Shuffle & Play All Tracks From'), self.album.name)
+            action.subtitle = _("Shuffle & Play All Tracks From '%s'") % self.album.name
             actions.append(action)
 
             action = TrackAddToFavoritesAction(self)
             action.untoggled_title = _('Add To Favorites')
-            action.untoggled_subtitle = "%s '%s' %s" % (_('Add The Album'), self.album.name, _('To Your Favorites'))
+            action.untoggled_subtitle = _("Add The Album '%s' To Your Favorites") % self.album.name
             action.toggled_title = _('Remove From Favorites')
-            action.toggled_subtitle = "%s '%s' %s" % (_('Remove The Album'), self.album.name, _('From Your Favorites'))
+            action.toggled_subtitle = _("Remove The Album '%s' From Your Favorites") % self.album.name
             actions.append(action)
 
-            # FIXME: add AND condition on foreign_class
             dfr = common.application.store.find(FavoritesItem,
-                                                FavoritesItem.foreign_id == self.album.name)
+                                                FavoritesItem.foreign_id == self.album.name,
+                                                FavoritesItem.foreign_class == u'MusicAlbum')
             dfr.addCallback(lambda rs: rs.all())
             dfr.addCallback(action.setup)
         elif self.track:
             action = TrackPlayAllAction(self)
             action.title = _('Play Track')
-            action.subtitle = "%s '%s'" % (_('Play'), self.track.title)
+            action.subtitle = _("Play '%s'") % self.track.title
             actions.append(action)
 
             action = TrackAddToFavoritesAction(self)
             action.untoggled_title = _('Add to Favorites')
-            action.untoggled_subtitle = "%s '%s' %s" % (_('Add'), self.track.title, _('To Your Favorites'))
+            action.untoggled_subtitle = _("Add '%s' To Your Favorites") % self.track.title
             action.toggled_title = _('Remove From Favorites')
-            action.toggled_subtitle = "%s '%s' %s" % (_('Remove'), self.track.title, _('From Your Favorites'))
+            action.toggled_subtitle = _("Remove '%s' From Your Favorites") % self.track.title
             actions.append(action)
 
-            # FIXME: add AND condition on foreign_class
             dfr = common.application.store.find(FavoritesItem,
-                                                FavoritesItem.foreign_id == self.track.file_path)
+                                                FavoritesItem.foreign_id == self.track.file_path,
+                                                FavoritesItem.foreign_class == u'MusicTrack')
             dfr.addCallback(lambda rs: rs.all())
             dfr.addCallback(action.setup)
         else:
@@ -656,6 +538,11 @@ class TracksDbController(GenericTracksDbController):
             result_set.order_by(MusicTrack.title)
             return result_set.all()
 
+        def sort_tracks_title_client_side(tracks):
+            trash_letters = " .,-_"
+            tracks.sort(key=lambda t: t.title.strip(trash_letters).lower())
+            return tracks
+
         def get_tracks(self):
             store = common.application.store
             if self.artist is not None:
@@ -663,7 +550,10 @@ class TracksDbController(GenericTracksDbController):
                 dfr = store.find(MusicTrack,
                                  MusicTrack.file_path == TrackArtist.track_path,
                                  TrackArtist.artist_name == self.artist.name)
+                # Add client side ordering - can this be done "server" side? Is
+                # there a better way to trash spurious characters?
                 dfr.addCallback(sort_tracks_title)
+                dfr.addCallback(sort_tracks_title_client_side)
             elif self.album is not None:
                 # FIXME: in this case we should not display an alphabetical
                 # fastscroller, which does not make sense for a small number of
@@ -673,7 +563,10 @@ class TracksDbController(GenericTracksDbController):
                 dfr.addCallback(sort_tracks_number_title)
             else:
                 dfr = store.find(MusicTrack, MusicTrack.title != None)
+                # Add client side ordering - can this be done "server" side? Is
+                # there a better way to trash spourious characters?
                 dfr.addCallback(sort_tracks_title)
+                dfr.addCallback(sort_tracks_title_client_side)
 
             dfr.addCallback(got_tracks)
             return dfr
@@ -731,14 +624,33 @@ class DBTracksViewMode(TracksViewMode):
         return super(DBTracksViewMode, self).get_preview_image(item, theme)
 
 
+class DBTracksInAlbumViewMode(DBTracksViewMode):
+
+    def get_label(self, item):
+        if isinstance(item, Action):
+            return defer.succeed(item.title)
+
+        # Strip trash letter - defined in TracksViewMode
+        title = item.title.strip(self.trash_letters)
+        if item.track_number:
+            title = '%s. %s' % (str(item.track_number).zfill(2), title)
+        return defer.succeed(title)
+
+
 class TracksPreviewListController(TracksDbController, DoubleLineMenuItemPreviewListController):
     view_mode = DBTracksViewMode
     fastscroller = True
 
-    def item_to_label(self, item):
-        if isinstance(item, Action):
-            return '#'
-        return item.title
+    def initialize(self, album=None, track=None, tracks=None, artist=None):
+        # Change the visualization of tracks, depending on whether they
+        # are displayed in one album or not
+        if album:
+            self.view_mode = DBTracksInAlbumViewMode
+
+        return super(TracksPreviewListController, self).initialize(album=album,
+                                                                   track=track,
+                                                                   tracks=tracks,
+                                                                   artist=artist)
 
 class TracksDbCoverflowController(TracksDbController, ImageWithReflectionCoverflowController):
     view_mode = DBTracksViewMode
@@ -899,6 +811,8 @@ class GenrePreviewListController(GenreDbController, DoubleLineMenuItemPreviewLis
     fastscroller = True
 
     def item_to_label(self, item):
+        if isinstance(item, Action):
+            return '#'
         return item.title
 
 class GenreDbCoverflowController(GenreDbController, ImageWithReflectionCoverflowController):
@@ -926,7 +840,7 @@ class DecadesDbController(HierarchyController):
             #        And what about other centuries?
             for decade in (50, 60, 70, 80, 90):
                 link = Link()
-                link.label = "%d's" % decade
+                link.label = _("%d's") % decade
                 link.icon = 'elisa.plugins.poblesec.by_decade'
                 link.controller_args = \
                     {'begin': datetime.datetime(1900 + decade, 1, 1),
@@ -1007,11 +921,28 @@ class TimeDbController(GenericAlbumsDbController):
                              (MusicAlbum.release_date <= end))
             return dfr
 
+        def missing_data(failure):
+            self.warning("Missing Data for decade.")
+            return self
+
         deferred.addCallback(get_albums)
         deferred.addCallback(sort_by_title)
         deferred.addCallback(got_albums)
+        deferred.addErrback(missing_data)
         return deferred
 
+    def nodes_setup(self):
+        super(TimeDbController, self).nodes_setup()
+        model = [m for m in self.model if not isinstance(m, Action)]
+        if len(model) == 0:
+            self.model[:] = []
+            msg = _("There are no albums for this decade.")
+            resource = 'elisa.plugins.poblesec.by_decade'
+            img = self.frontend.get_theme().get_resource(resource)
+            self.nothing_to_display_widget.icon.set_from_file(img)
+            self.nothing_to_display_widget.text.label = msg
+            self.nothing_to_display_widget.visible = True
+
 
 class TimeDbPreviewListController(TimeDbController, MenuItemPreviewListController):
     view_mode = AlbumsDbViewMode
