@@ -22,19 +22,18 @@ from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController
 
 from elisa.plugins.database.photo_controller import PhotoVerticalWithPreview, \
     PhotoCoverflow, PhotoGrid
+from elisa.plugins.database.photo_controller import GenericPhotosController
 
 from elisa.plugins.poblesec.base.preview_list import \
-    DoubleLineMenuItemPreviewListController
+    MenuItemPreviewListController, DoubleLineMenuItemPreviewListController
 from elisa.plugins.poblesec.base.coverflow import \
     ImageWithReflectionCoverflowController
 from elisa.plugins.poblesec.base.grid import GridItemGridController
 
-
-
 from elisa.plugins.database.music_controller import GenericTracksDbController, \
                                               DBTracksViewMode
 
-from elisa.plugins.database.photo_controller import PhotoController, \
+from elisa.plugins.database.photo_controller import PhotosController, \
                                               PhotoViewMode
 from elisa.plugins.poblesec.link import Link
 
@@ -130,6 +129,7 @@ def pictures_playlists_decorator(controller):
 
     return defer.succeed(None)
 
+
 class PlayedMixin(object):
     def _load(self, result, filter, order_by, limit):
         self.filter = filter
@@ -215,14 +215,14 @@ class AddedTracksController(GenericTracksDbController, AddedMixin):
         dfr.addCallback(self._load, MusicTrack, limit)
         return dfr
 
-class PlayedPhotosController(PhotoController, PlayedMixin):
+class PlayedPhotosController(GenericPhotosController, PlayedMixin):
     def initialize(self, order_by=None, limit=10):
         dfr = super(PlayedPhotosController, self).initialize()
 
         dfr.addCallback(self._load, Image, order_by, limit)
         return dfr
 
-class AddedPhotosController(PhotoController, AddedMixin):
+class AddedPhotosController(GenericPhotosController, AddedMixin):
 
     def initialize(self, limit=10):
         dfr = super(AddedPhotosController, self).initialize()
@@ -264,13 +264,13 @@ class PlayedTracksListSwitcherController(ListSwitcherController):
     default_mode = PlayedTracksVerticalWithPreview
 
 # Added Photos 
-class AddedPhotosVerticalWithPreview(AddedPhotosController, PhotoVerticalWithPreview):
+class AddedPhotosVerticalWithPreview(AddedPhotosController, MenuItemPreviewListController):
     view_mode = PhotoViewMode
 
-class AddedPhotosCoverflow(AddedPhotosController, PhotoCoverflow):
+class AddedPhotosCoverflow(AddedPhotosController, ImageWithReflectionCoverflowController):
     view_mode = PhotoViewMode
 
-class AddedPhotosGrid(AddedPhotosController, PhotoGrid):
+class AddedPhotosGrid(AddedPhotosController, GridItemGridController):
     view_mode = PhotoViewMode
 
 class AddedPhotosListSwitcherController(ListSwitcherController):
@@ -279,14 +279,14 @@ class AddedPhotosListSwitcherController(ListSwitcherController):
              AddedPhotosGrid]
     default_mode = AddedPhotosGrid
 
-# Played Photos 
-class PlayedPhotosVerticalWithPreview(PlayedPhotosController, PhotoVerticalWithPreview):
+# Played Photos
+class PlayedPhotosVerticalWithPreview(PlayedPhotosController, MenuItemPreviewListController):
     view_mode = PhotoViewMode
 
-class PlayedPhotosCoverflow(PlayedPhotosController, PhotoCoverflow):
+class PlayedPhotosCoverflow(PlayedPhotosController, ImageWithReflectionCoverflowController):
     view_mode = PhotoViewMode
 
-class PlayedPhotosGrid(PlayedPhotosController, PhotoGrid):
+class PlayedPhotosGrid(PlayedPhotosController, GridItemGridController):
     view_mode = PhotoViewMode
 
 class PlayedPhotosListSwitcherController(ListSwitcherController):
