@@ -1,3 +1,19 @@
+# -*- coding: utf-8 -*-
+# Elisa - Home multimedia server
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
+# All rights reserved.
+#
+# This file is available under one of two license agreements.
+#
+# This file is licensed under the GPL version 3.
+# See "LICENSE.GPL" in the root of this distribution including a special
+# exception to use Elisa with Fluendo's plugins.
+#
+# The GPL part of Elisa is also available under a commercial licensing
+# agreement from Fluendo.
+# See "LICENSE.Elisa" in the root directory of this distribution package
+# for details on that license.
+
 import os
 from setuptools import setup
 from elisa.core.utils.dist import find_packages, TrialTest, Clean
@@ -18,6 +34,9 @@ setup(name='elisa-plugin-database',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
+    package_data={'': ['*.png', '*.mo', '*.po', '*.conf'],
+                  'elisa.plugins.database.i18n': ['*/LC_MESSAGES/*.po', '*.pot'],
+                 },
     entry_points="",
     controller_mappings=[
                          # FIXME: we should create a controller path convention
@@ -42,6 +61,8 @@ setup(name='elisa-plugin-database',
                           'elisa.plugins.database.playlists_controller:AddedTracksListSwitcherController'),
                          ('/poblesec/database/photo/list',
                           'elisa.plugins.database.photo_controller:PhotoListSwitcherController'),
+                         ('/poblesec/database/photo/photo_actions',
+                          'elisa.plugins.database.photo_controller:PhotoVerticalWithPreview'),
                          ('/poblesec/database/photo/albums',
                           'elisa.plugins.database.photo_controller:AlbumsListSwitcherController'),
                          ('/poblesec/database/photo/photos_added',
