@@ -44,12 +44,14 @@ class GstMetadataMaster(Loggable, Master):
     serverFactory = GstMetadataMasterFactory
     slaveProcessProtocolFactory = GstMetadataSlaveProcessProtocol 
     socket_prefix = 'elisa-metadata-'
+    max_restart_retry = 10
 
     def __init__(self):
         Loggable.__init__(self)
         Master.__init__(self, 
                 slave_runner='elisa.plugins.gstreamer.amp_slave.run_slave')
         self._requests = RequestQueue()
+        self._restarted = 0
         self._next_request_call = None
         self._in_request = False
         self._process_next_request = None
@@ -57,11 +59,30 @@ class GstMetadataMaster(Loggable, Master):
 
     def _restart_callback(self, result):
         self._restart_defer = None
+        self._restarted = 0
         self._schedule_next_request()
 
+    def slaveDead(self, slave):
+        if self._restart_defer or self._stop_defer:
+            # we are in the restart process, so we let the _restart_callbacks
+            # act or we are about to stop so we wanted the processes to end
+            return
+        self._restart_slave()
+
     def _restart_errback(self, failure):
         self._restart_defer = None
-        self.warning('could not restart slave %s' % getFailureMessage(failure))
+
+        self.info('could not restart slave %s' % getFailureMessage(failure))
+
+        self._restarted += 1
+        if self._restarted < self.max_restart_retry:
+            self._restart_slave()
+            return
+
+        self._restarted = 0
+
+        self.warning('Even after %s restarts the slave still fails: %s' % \
+                    ( self.max.restart_retry, getFailureMessage(failure)))
 
         while True:
             try:
@@ -93,6 +114,13 @@ class GstMetadataMaster(Loggable, Master):
         assert not self._in_request
         self._next_request_call = None
 
+        try:
+            slave = self._get_slave()
+        except IndexError:
+            # no slave running, we try to restart one first
+            self._restart_slave()
+            return
+
         try:
             request = self._requests.dequeue()
         except IndexError:
@@ -101,17 +129,20 @@ class GstMetadataMaster(Loggable, Master):
 
         self.debug('starting request %s' % request.metadata)
 
-        slave = self._get_slave()
-        protocol = slave.amp
+        try:
+            protocol = slave.amp
 
-        keys = [{'key': key} for key in request.metadata.keys()
-                if key != 'uri']
+            keys = [{'key': key} for key in request.metadata.keys()
+                    if key != 'uri']
 
-        dfr = protocol.callRemote(GetMetadata, 
-                uri=str(request.metadata['uri']), metadata=keys)
-        self._in_request = True
-        dfr.addCallback(self._get_metadata_cb, request)
-        dfr.addErrback(self._get_metadata_eb, request)
+            dfr = protocol.callRemote(GetMetadata, 
+                    uri=str(request.metadata['uri']), metadata=keys)
+            self._in_request = True
+            dfr.addCallback(self._get_metadata_cb, request)
+            dfr.addErrback(self._get_metadata_eb, request)
+        except:
+            if not request.cancelled:
+                request.defer.errback()
 
     def _get_metadata_cb(self, result, request):
         self._in_request = False
@@ -122,6 +153,13 @@ class GstMetadataMaster(Loggable, Master):
         self._schedule_next_request()
         request.defer.callback(request.metadata)
 
+    def _restart_slave(self):
+        self.warning('restarting slave')
+
+        self._restart_defer = self.startSlaves(1)
+        self._restart_defer.addCallback(self._restart_callback)
+        self._restart_defer.addErrback(self._restart_errback)
+
     def _get_metadata_eb(self, failure, request):
         self._in_request = False
 
@@ -133,11 +171,7 @@ class GstMetadataMaster(Loggable, Master):
         # windows systems. For now we restart in any case but later we should
         # find out why it happens in the first place and fix it there.
         if failure.check(ConnectionDone, ConnectionLost) and self._slaves_num:
-            self.warning('restarting slave')
-
-            self._restart_defer = self.startSlaves(1)
-            self._restart_defer.addCallback(self._restart_callback)
-            self._restart_defer.addErrback(self._restart_errback)
+            self._restart_slave()
         else:
             self._schedule_next_request()
 
