@@ -269,7 +269,8 @@ class GstMetadataPipeline(Loggable):
             self.debug('reached PAUSED')
 
             if self._video_caps is None and self._image_caps is None and \
-                self._typefind_file_type not in ('video', 'image'):
+                self._typefind_file_type not in ('video', 'image') or \
+                self._have_thumbnail():
                 # we have the tags at this point
                 self._done()
 
@@ -299,12 +300,14 @@ class GstMetadataPipeline(Loggable):
 
         if req_keys == media_type_keys and file_type in \
                     ('video', 'audio', 'image') or \
-                file_type == 'video' and \
+                (file_type == 'video' and \
                     (not 'thumbnail' in req_keys or  \
-                    self._have_thumbnail()) or \
+                    self._have_thumbnail())) or \
                 (file_type == 'image' and \
-                    req_keys.intersection(supported_keys) == thumbnail_keys \
-                    and  self._have_thumbnail()):
+                    req_keys.intersection(supported_keys) in 
+                            (thumbnail_keys,
+                            media_type_keys.union(thumbnail_keys))
+                            and  self._have_thumbnail()):
 
             self.debug('got media_type for %s, NOT going to paused',
                     self._req_uri)
@@ -313,7 +316,7 @@ class GstMetadataPipeline(Loggable):
             structure = gst.Structure('metadata-done')
             self._bus.post(gst.message_new_application(self._pipeline, structure))
             return
-
+        
         # we need tags and/or a thumbnail
         self.debug('we need to go to PAUSED, plugging decodebin '
                 '(file_type: %s, have_thumbnail: %s)', file_type,
@@ -532,7 +535,9 @@ class GstMetadataPipeline(Loggable):
                 self._video_caps = caps
                 # do a thumbnail of the first video track
                 self._plug_video_thumbnailbin(pad)
-        elif typ == 'image' and "thumbnail" in self._req_metadata:
+        elif typ == 'image' and \
+                ('thumbnail' in self._req_metadata and not
+                self._have_thumbnail()):
             if self._image_caps is None:
                 self._image_caps = caps
                 self._plug_image_thumbnailbin(pad)
