@@ -159,6 +159,10 @@ class HTTPParser(object):
         if datalen < self.length:
             self.handleContentChunk(data)
             self.length = self.length - datalen
+        elif self.length is None:
+           # no content-length header specified and not chunking, we just pass
+           # it through:
+           self.handleContentChunk(data)
         else:
             self.handleContentChunk(data[:self.length])
             extraneous = data[self.length:]
@@ -595,6 +599,11 @@ class HTTPChannelRequest(HTTPParser):
 
     def connectionLost(self, reason):
         """connection was lost"""
+        if self.length is None:
+            # as there was no length specified every thing we got up to this
+            # point must be treated as the full content:
+            self.handleContentComplete()
+
         if self.queued and self.producer:
             self.producer.stopProducing()
             self.producer = None
