@@ -15,9 +15,9 @@
 # for details on that license.
 
 from twisted.trial.unittest import TestCase
-from twisted.internet import reactor, defer, error
+from twisted.internet import reactor, defer, error, protocol
 from twisted.web2.http import StatusResponse
-from twisted.web2.channel.http import HTTPFactory
+from twisted.web2.channel.http import HTTPFactory, HTTPChannel
 from twisted.web2 import iweb, server
 from twisted.web2 import responsecode
 from twisted.web2.stream import BufferedStream, MemoryStream
@@ -197,9 +197,187 @@ class SetupServerMixin(object):
         # Stop listening
         return defer.maybeDeferred(self._port.stopListening)
 
+class RawDataProtocol(HTTPChannel):
+    first_line = None
+    req_to_answer = {'GET /no_size HTTP/1.1' : (
+                        "HTTP/1.1 200 OK\r\n"\
+                        "Date: Wed, 27 Aug 2008 08:52:03 GMT\r\n" \
+                        "Server: Apache/2.2.3 (CentOS)\r\n" \
+                        "X-Powered-By: PHP/5.1.6\r\n" \
+                        "Cache-Control: max-age=0, must-revalidate," \
+                                "no-transform,private\r\n" \
+                        "Vary: Accept-Encoding,User-Agent\r\n" \
+                        "Connection: close\r\n" \
+                        "Content-Type: application/json\r\n" \
+                        "\r\n\r\n" + \
+                        ' '.join(['test%s' % x for x in xrange(10000)]),
+                        True),
+                    'HEAD /some_thing HTTP/1.1' : (
+                        "HTTP/1.1 200 OK\r\n"\
+                        "Date: Wed, 27 Aug 2008 08:52:03 GMT\r\n" \
+                        "Server: Apache/2.2.3 (CentOS)\r\n" \
+                        "X-Powered-By: PHP/5.1.6\r\n" \
+                        "Cache-Control: max-age=0, must-revalidate," \
+                                "no-transform,private\r\n" \
+                        "Vary: Accept-Encoding,User-Agent\r\n" \
+                        "\r\n", False),
+                    'GET /no_content HTTP/1.1' : (
+                        "HTTP/1.1 204 NO CONTENT\r\n"\
+                        "Date: Wed, 27 Aug 2008 08:52:03 GMT\r\n" \
+                        "Server: Apache/2.2.3 (CentOS)\r\n" \
+                        "X-Powered-By: PHP/5.1.6\r\n" \
+                        "Cache-Control: max-age=0, must-revalidate," \
+                                "no-transform,private\r\n" \
+                        "Vary: Accept-Encoding,User-Agent\r\n" \
+                        "\r\n", False),
+                    'GET /not_modified HTTP/1.1' : (
+                        "HTTP/1.1 304 NOT MODIFIED\r\n"\
+                        "Date: Wed, 27 Aug 2008 08:52:03 GMT\r\n" \
+                        "Server: Apache/2.2.3 (CentOS)\r\n" \
+                        "X-Powered-By: PHP/5.1.6\r\n" \
+                        "Cache-Control: max-age=0, must-revalidate," \
+                                "no-transform,private\r\n" \
+                        "Vary: Accept-Encoding,User-Agent\r\n" \
+                        "\r\n", False),
+                }
+    def lineReceived(self, line):
+        if self.first_line is None:
+            self.first_line = line
+        if line != '':
+            return
+        try:
+            data, close = self.req_to_answer[self.first_line]
+        except KeyError:
+            # not found. return nothing and close it
+            data = 'HTTP/1.1 404 NOT FOUND\r\n\r\n'
+            close = True
+
+        self.transport.write(data)
+        if close:
+            self.transport.loseConnection()
+
+class RawDataFactory(HTTPFactory):
+    """Factory for HTTP server."""
+
+    protocol = RawDataProtocol
+    
+    protocolArgs = None
+
+    outstandingRequests = 0
+    
+    def __init__(self):
+        HTTPFactory.__init__(self, DummyHttpServerRequest)
 
 # Actual tests on the HTTP client.
 
+class TestElisaHttpClientRawData(TestCase):
+    """
+    Tests that need the RawServer as they try to ensure the right behaviour
+    according to the message length. See the RFC for more information:
+    http://www.w3.org/Protocols/rfc2616/rfc2616-sec4.html#sec4.4
+    """
+
+    def setUp(self):
+        # Start listening
+        self.http_factory = RawDataFactory()
+        self._port = reactor.listenTCP(SERVER_PORT, self.http_factory)
+        self.client = ElisaHttpClient(SERVER_HOST, SERVER_PORT)
+
+    def tearDown(self):
+        # Stop listening
+        dfr = self.client.close()
+        dfr.addCallback(lambda x: defer.maybeDeferred(
+                self._port.stopListening))
+        return dfr
+
+    def test_large_response_contents_without_specified_size(self):
+        """
+        Test that a 'large' resource is completely read even though the size is
+        not specified.
+        """
+        resource = 'no_size'
+
+        def response_read(response):
+            # check if we really got all the data we wanted to have
+            end = response[-8:]
+            self.assertEqual(end, 'test9999')
+
+        def request_done(response):
+            self.assertEqual(response.code, responsecode.OK)
+            # Read the response's contents
+            read_dfr = BufferedStream(response.stream).readExactly()
+            read_dfr.addCallback(response_read)
+            return read_dfr
+
+        request_dfr = self.client.request('/' + resource)
+        request_dfr.addCallback(request_done)
+        return request_dfr
+
+    def test_head_does_not_wait_for_content(self):
+        """
+        Sending a head to the server. As soon as the headers (and the empty
+        line) are back the deferred should fire.
+        """
+
+        def response_read(response):
+            # check if we really got all the data we wanted to have
+            self.assertEqual(response, '')
+
+        def request_done(response):
+            self.assertEqual(response.code, responsecode.OK)
+            # Read the response's contents
+            read_dfr = BufferedStream(response.stream).readExactly()
+            read_dfr.addCallback(response_read)
+            return read_dfr
+
+        request = ClientRequest('HEAD', '/some_thing', None, None)
+    
+        request_dfr = self.client.request_full(request)
+        request_dfr.addCallback(request_done)
+        return request_dfr
+
+    def test_no_content_does_not_wait_for_content(self):
+        """
+        A request that is answered with "no content". As soon as the headers are
+        here the callback should fire and there should not be any content at
+        all.
+        """
+        def response_read(response):
+            # check if we really got all the data we wanted to have
+            self.assertEqual(response, '')
+
+        def request_done(response):
+            self.assertEqual(response.code, 204)
+            # Read the response's contents
+            read_dfr = BufferedStream(response.stream).readExactly()
+            read_dfr.addCallback(response_read)
+            return read_dfr
+
+        request_dfr = self.client.request('/no_content')
+        request_dfr.addCallback(request_done)
+        return request_dfr
+
+    def test_not_modified_does_not_wait_for_content(self):
+        """
+        A request that is answered with a "not modified". As soon as the headers
+        are here the callback should fire and there should be no content at all.
+        """
+        def response_read(response):
+            # check if we really got all the data we wanted to have
+            self.assertEqual(response, '')
+
+        def request_done(response):
+            self.assertEqual(response.code, 304)
+            # Read the response's contents
+            read_dfr = BufferedStream(response.stream).readExactly()
+            read_dfr.addCallback(response_read)
+            return read_dfr
+
+        request_dfr = self.client.request('/not_modified')
+        request_dfr.addCallback(request_done)
+        return request_dfr
+
+
 class TestElisaHttpClientConnection(TestCase, SetupServerMixin):
 
     """
@@ -999,7 +1177,6 @@ class TestElisaAdvancedHttpClient(TestCase, SetupServerMixin):
         result_dfr = defer.Deferred()
         request_dfr = self.client.request('/double_redirect')
         request_dfr.addCallback(self.request_done)
-        request_dfr.addErrback(result_dfr.errback)
         request_dfr.chainDeferred(result_dfr)
         return result_dfr
 
@@ -1011,7 +1188,6 @@ class TestElisaAdvancedHttpClient(TestCase, SetupServerMixin):
         result_dfr = defer.Deferred()
         request_dfr = self.client.request('/redirect_and_close')
         request_dfr.addCallback(self.request_done)
-        request_dfr.addErrback(result_dfr.errback)
         request_dfr.chainDeferred(result_dfr)
         return result_dfr
 
