@@ -506,7 +506,7 @@ class Player(gobject.GObject, Loggable):
         pbus.add_signal_watch()
 
     def _message_eos_cb(self, bus, message):
-        self.pipeline.set_state(gst.STATE_READY)
+        self.pipeline.set_state(gst.STATE_NULL)
         self.play_next()
 
     def _message_buffering_cb(self, bus, message):
@@ -517,7 +517,7 @@ class Player(gobject.GObject, Loggable):
         err, msg = message.parse_error()
         self.last_error = (err, msg)
         self.warning("Gstreamer %s:%s" % (err, msg))
-        self.pipeline.set_state(gst.STATE_READY)
+        self.pipeline.set_state(gst.STATE_NULL)
         code = message.structure['gerror'].code
         if code == gst.STREAM_ERROR_CODEC_NOT_FOUND:
             self.emit('player-codec-error')
@@ -638,16 +638,15 @@ class Player(gobject.GObject, Loggable):
                 visu = ElisaVisualisationBin(vis_plugin_name, 400,
                                              self.image)
                 self.pipeline.set_property('vis-plugin', visu)
-            except:
+            except gst.ElementNotFoundError:
                 pass
 
-
             self.play()
             self._update_playcount_and_time(model)
             self.current_index = index
 
     def stop(self):
-        self.pipeline.set_state(gst.STATE_READY)
+        self.pipeline.set_state(gst.STATE_NULL)
         if self.image:
             self.image.clear()
 
@@ -655,7 +654,7 @@ class Player(gobject.GObject, Loggable):
         self.pipeline.set_state(gst.STATE_PAUSED)
 
     def play(self):
-        current = self.pipeline.get_state()[1]
+        current = self.pipeline.get_state(0)[1]
         if current != gst.STATE_PLAYING:
             if current != gst.STATE_PAUSED:
                 self.status = self.BUFFERING
@@ -666,30 +665,27 @@ class Player(gobject.GObject, Loggable):
         try:
             position, format = self.pipeline.query_position(gst.FORMAT_TIME)
             return position
-        except:
+        except gst.QueryError:
             return -1
 
     def set_position(self, value):
         if self.status not in (self.PLAYING, self.PAUSED):
             return
 
-        try:
-            event = self.pipeline.seek(1.0, gst.FORMAT_TIME,
-                                gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_KEY_UNIT,
-                                gst.SEEK_TYPE_SET, value,
-                                gst.SEEK_TYPE_SET, -1 )
-        except:
-            pass
+        event = self.pipeline.seek(1.0, gst.FORMAT_TIME,
+                            gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_KEY_UNIT,
+                            gst.SEEK_TYPE_SET, value,
+                            gst.SEEK_TYPE_SET, -1 )
 
         duration = self.get_duration()
-        if value >= duration:
+        if duration != -1 and value >= duration:
             self.play_next()
 
     def get_duration(self):
         try:
             duration, format = self.pipeline.query_duration(gst.FORMAT_TIME)
             return duration
-        except:
+        except gst.QueryError:
             return -1
 
     def _can_handle_key(self):
@@ -727,7 +723,7 @@ class Player(gobject.GObject, Loggable):
     def seek_backward(self):
         duration = self.get_duration()
         position = self.get_position()
-        if duration > 0 and self._can_handle_key():
+        if position != -1 and duration > 0 and self._can_handle_key():
             position -= self.seek_backward_seconds * gst.SECOND
             if position < 0:
                 # the user wants to seek to the previous track
@@ -740,12 +736,14 @@ class Player(gobject.GObject, Loggable):
     def seek_forward(self):
         duration = self.get_duration()
         position = self.get_position()
-        if duration > 0 and self._can_handle_key():
+        if position != -1 and duration > 0 and self._can_handle_key():
             position += self.seek_forward_seconds * gst.SECOND
             self.set_position(position)
         return position
 
 class PlayerController(PigmentController):
+    
+    PlayerClass = Player
 
     def __init__(self):
         super(PlayerController, self).__init__()
@@ -778,7 +776,7 @@ class PlayerController(PigmentController):
         self._rotation_matrix.translate(-0.5, -0.5, 0.0)
         self._create_interaction()
 
-        self.player = Player()
+        self.player = self.PlayerClass()
         self.player.set_drawable(self.background)
         self.player.connect('status-changed',self._player_status_cb)
 
@@ -940,15 +938,10 @@ class PlayerController(PigmentController):
             else:
                 text = _('Unfortunately Elisa cannot play the selected file.')
             if missing_decoders:
-                text += _(' Please download and install a ')
+                text += _(' Please download and install: ')
                 # Localization of the names of the missing decoders is
                 # automatically taken care of by gstreamer.
-                if len(missing_decoders) == 1:
-                    decoders_string = missing_decoders[0]
-                else:
-                    decoders_string = _(', a ').join(missing_decoders[:-1])
-                    decoders_string += _(' and a ') + missing_decoders[-1]
-                text += decoders_string + _('.')
+                text += _(', ').join(missing_decoders) + _('.')
 
             buttons = [(_('Close'), main.hide_popup)]
 
@@ -1112,3 +1105,4 @@ class PlayerController(PigmentController):
             self.toggle_mute()
 
         return False
+
