@@ -82,21 +82,24 @@ setup(name='elisa-plugin-poblesec',
                         ('/poblesec/sections_menu',
                          'elisa.plugins.poblesec.music:music_decorator'),
                         ('/poblesec/music_menu',
-                         'elisa.plugins.poblesec.music:hardware_decorator'),
+                         'elisa.plugins.poblesec.section:hardware_decorator'),
+                        ('/poblesec/music_menu',
+                         'elisa.plugins.poblesec.music:internet_decorator'),
+                        ('/poblesec/music_menu',
+                         'elisa.plugins.poblesec.music:network_decorator'),
+                        # FIXME: the following decorator should not be here
+                        # but in the database plugin ; this works around an
+                        # ordering issue in the menus.
+                        ('/poblesec/music_menu',
+                         'elisa.plugins.database.playlists_controller:music_playlists_decorator'),
                         # FIXME: the following decorator should not be here
                         # but in the favorites plugin ; this works around an
                         # ordering issue in the menus.
                         ('/poblesec/music_menu',
                          'elisa.plugins.favorites.controller:music_decorator'),
-                        ('/poblesec/music_menu',
-                         'elisa.plugins.poblesec.music:internet_decorator'),
-                        ('/poblesec/music_menu',
-                         'elisa.plugins.poblesec.music:network_decorator'),
-                        # FIXME: the following 2 decorators should not be here
+                        # FIXME: the following decorator should not be here
                         # but in the database plugin ; this works around an
                         # ordering issue in the menus.
-                        ('/poblesec/music_menu',
-                         'elisa.plugins.database.playlists_controller:music_playlists_decorator'),
                         ('/poblesec/music_menu',
                          'elisa.plugins.database.music_controller:music_lib_decorator'),
                         ('/poblesec/music_menu',
@@ -104,7 +107,7 @@ setup(name='elisa-plugin-poblesec',
                         ('/poblesec/music_menu',
                          'elisa.plugins.poblesec.music:albums_decorator'),
                         ('/poblesec/video_menu',
-                         'elisa.plugins.poblesec.video:hardware_decorator'),
+                         'elisa.plugins.poblesec.section:hardware_decorator'),
                         ('/poblesec/video_menu',
                          'elisa.plugins.poblesec.video:internet_decorator'),
                         ('/poblesec/video_menu',
@@ -112,12 +115,17 @@ setup(name='elisa-plugin-poblesec',
                         ('/poblesec/video_menu',
                          'elisa.plugins.poblesec.video:filesystem_decorator'),
                         ('/poblesec/pictures_menu',
-                         'elisa.plugins.poblesec.pictures:hardware_decorator'),
+                         'elisa.plugins.poblesec.section:hardware_decorator'),
                         ('/poblesec/pictures_menu',
                          'elisa.plugins.poblesec.pictures:internet_decorator'),
                         ('/poblesec/pictures_menu',
                          'elisa.plugins.poblesec.pictures:network_decorator'),
                         # FIXME: the following decorator should not be here
+                        # but in the favorites plugin ; this works around an
+                        # ordering issue in the menus.
+                        ('/poblesec/pictures_menu',
+                        'elisa.plugins.favorites.controller:pictures_decorator'),
+                        # FIXME: the following decorator should not be here
                         # but in the database plugin ; this works around an
                         # ordering issue in the menus.
                         ('/poblesec/pictures_menu',
